{% extends "base_layout.jinja" %}
{% set active_page = "profile" %}
{% import 'macros_view.jinja' as csv %}



{% block content %}
    <div class="ui secondary segment container ">
        <h2 class="ui divider horizontal header">
            <div class="content">
                CSV Profile
                <div class="sub header">Compute the profile of a CSV file</div>
            </div>
        </h2>
    </div>
    <div class="ui segment container ">
        <div class="ui one column centered grid ">
            <div class="column">
                <div class="ui fluid card" width="100%">
                {{ csv.display_table_card_header( table.profiledata.table_metadata ) }}
                {{ csv.display_table_dialect(table.profiledata.table_metadata) }}
                {{ csv.display_table_card_data( table, table.sample ) }}
                {{ csv.display_table_fd( table.profiledata) }}
                {{ csv.display_table_colum_profile( table.profiledata ) }}
                </div>
            </div>
        </div>
        </div>


{% endblock %}


{% block script %}
        {{csv.scripts()}}

{% endblock %}
